/;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;;  Welcome to Ardour
;;;  -------------------------------------------
;;;
;;;  This is a sample configuration file. It contains specifications
;;;  and examples for all possible settings.  This file is loaded by
;;;  'guile', the GNU extension language.  guile supports the 'scheme'
;;;  dialect of LISP as defined by the Revised^5 Report On The
;;;  Algorithmic Language "Scheme" (R5RS).  See
;;;  http://www.gnu.org/software/guile/guile.html for more details.
;;;
;;;  All the simple (set-config-var 'name value) assignments in
;;;  this file can be overridden on the command line using the
;;;  variable name preceeded by "--" (i.e. GNU-style long options).
;;;;    ( XXX Not yet! This feature coming, well, err, soon-ish)
;;;   
;;;  Comments are either lisp-style (; to end of line) or block-style
;;;  ( #| up to |#)
;;;
;;;  $Id: ardourrc.scm,v 1.47 2000/11/03 04:25:18 pbd Exp $
;;;
;;;  Copyright (C) 1999 Bill Gribble
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;; the systemic-latency is the number of samples of delay between the
;; output of your audio interface h/w and the analog
;; speaker(s) used to play the signal. In a system where this
;; connection is purely analog, the correct setting is zero - there is
;; no (reasonably measurable) delay. In a system with digital outputs
;; going to a a digital mixer, the delay is non-zero, and will have to
;; measured.  Doing so is not easy, but its not hard either, and I
;; should write up a description of how to do it.
;;
;; This is important because it ensures that material that is overdubbed
;; with the intent of fitting together with "what i am hearing right now"
;; ends up at the correct spot on the "tape".

(set-config-var systemic-latency 0)

;; max-tracks mostly just affects the display. set it a value equal
;; to the larger of:
;;
;;     * the largest number of tracks on a "tape"
;;     * the number of channels on your audio interface h/w.

(set-config-var max-tracks 26)

;; Since ardour has to display and send timecode information, which is
;; done using (stupid) SMTPE conventions, you have to tell ardour what
;; SMTPE frame rate you want it to use for the calculation of timecode.

(set-config-var smpte-frames-per-second 30)

;; This figure should probably not be changed. It specifies the block
;; size that ardour uses the write/read data to/from disk. 256kB appears
;; to optimum for the Linux ext2 filesystem on all disks.

(set-config-var minimum-disk-io-bytes 262144)

;; This figure should probably not be changed. It specifies how much
;; of the data from the tracks should be held in memory at all times.
;; It needs to be large enough to cover jitter in the data rate coming
;; from the disk, and not so large as to use absurd amount of memory.
;; How much memory ? Well, 1 track at 48kHz for 5 seconds is about 1MB.
;; Experiments by myself and Benno Sennoner suggest that 5 seconds is
;; about right.

(set-config-var track-buffer-seconds 5.0)

;; Should Ardour try to run with realtime priority, and if so, at
;; what priority ?
;;
;; This requires root permissions at run time in order to be
;; effective. However, ardour will not function effectively without it
;; being set to "#t" unless you use a large frames-per-cycle value and
;; do not work with more than a few tracks.

(set-config-var use-real-time #t)
(set-config-var real-time-priority 10)

;;
;; MIDI port specs
;;
;; type can be: OSS/RAW, ALSA/RAW, ALSA/SEQUENCER, IRIX/RAW, NULL, FIFO
;; mode can be: input, output, duplex (aka inout)
;; you must quote them, and they can be upper case or lower case but not
;; mixed case (FIX ME!!)
;;
;; note: FIFO cannot be duplex, but this is currently unchecked!
;;

(define midiport-first
  (make-mididev 
   '((device: "/dev/snd/midiC0D0")
     (tag: "first")
     (type: "alsa/raw")
     (mode: "duplex"))))

(set-config-var midi-device midiport-first)

;;
;; Audio interface specifications
;;
;; ALSA devices are always specified with in the form:
;; 
;;      card:pcm-device:pcm-subdevice
;;
;;

(define soundcard-hammerfall
  (make-soundcard
   '((device: "hammerfall")

;; the rest of the settings here are ignored
;; because ardour is an ALSA-only program
;; and always opens the audio interfaces duplex.

     (format: "linear_little_endian32")
     (type: "ALSA")
     (mode: "duplex"))))

(define soundcard-trident
  (make-soundcard
   '((device: "trident")

     (format: "linear_little_endian16")
     (type: "ALSA")
     (mode: "duplex"))))

(define soundcard-tropez+
  (make-soundcard
   '((device: "tropez")

     (format: "linear_little_endian16")
     (type: "ALSA")
     (mode: "duplex"))))

(define soundcard-hammerfall2
  (make-soundcard
   '((device: "hammerfall2")

     (format: "linear_little_endian32")
     (type: "ALSA")
     (mode: "duplex"))))

;;============================================================================
;; Audio interface
;;============================================================================

(set-config-var audio-device soundcard-hammerfall)
;;(set-config-var audio-device soundcard-trident)
;;(set-config-var audio-device soundcard-tropez+)

;; !!! this will be removed soon !!!

(set-config-var sample-rate 48000)

;; the number of samples (frames) you want the audio interface to
;; process before it tells the "tape" to do something. This controls
;; the overall latency in ardour. Setting it a large value reduces
;; system load; a small value gives better latency at the expense of
;; somewhat higher load. Values should be powers-of-2, in the range 64
;; to 8192, though these limits are somewhat dependent upon your audio
;; interface h/w.

(set-config-var frames-per-cycle 256)

;; directory for tapes

(set-config-var tape-path "/tape1")

;; The full name of the Ardour GUI rc file (which contains font, color and
;; other settings for the GUI

(set-config-var ui-rcfile "/usr/local/music/src/ardour/ardour_ui.rc")

;; where to look for pixmaps (a colon-separated search path)

(set-config-var pixmap-path "/usr/local/music/src/ardour/pixmaps")

;; regular expressions for certain sets of pixmaps.
;; these will be found using the Ardour pixmap-path 

(set-config-var rec-enable-pix-regexp "^rec-enable-button[0-9]*.xpm$")
(set-config-var monitor-pix-regexp "^monitor-button[0-9]*.xpm$")
(set-config-var toggle-button-pix-regexp    "^toggle-button-[0-9]*.xpm$")
(set-config-var push-button-pix-regexp    "^round-button-[0-9]*.xpm$")
(set-config-var small-push-button-pix-regexp    "^small-round-button-[0-9]*.xpm$")

;; specific pixmaps used for transport and other buttons with no state.
;; these will be found using the GTK pixmap_path

(set-config-var tape-button-image      "tape-button.xpm")
(set-config-var play-button-image      "play-button.xpm")
(set-config-var forward-button-image   "fwd-button.xpm")
(set-config-var reverse-button-image   "rev-button.xpm")
(set-config-var record-button-image    "rec-button.xpm")
(set-config-var stop-button-image      "stop-button.xpm")
(set-config-var start-button-image     "start-button.xpm")
(set-config-var end-button-image       "end-button.xpm")
(set-config-var set-next-button-image  "set-next-button.xpm")

;; Set this if you want to load a particular tape everytime ardour
;; starts up. Its just the name of the tape, as displayed in ardour's
;; GUI.

(set-config-var default-tape "foo")

;; You can turn off MTC transmission if you don't want/need it. Its
;; on by default. This can also be controlled from the UI.

(set-config-var send-midi-timecode #t)

;; If your MMC-controller sends a range of shuttle speeds that are
;; not particularly useful (e.g. the Mackie D8B sends only 0.0625-6)
;; you can scale the shuttle speed here. The threshold value
;; specifies the lowest MMC-supplied shuttle speed to which scaling
;; should be applied; the `factor' is used to multiply all such values.
;; 
;; Set the `threshold' to -1 to never do scaling. Both variables have
;; floating point values, so you can scale all values above 0.7189 by
;; 3.14159 if you want to.

(set-config-var shuttle-speed-threshold 1)
(set-config-var shuttle-speed-factor 5)

;; This is the number of samples that the tape moves by during a single
;; "rewind" or "fast forward" operation. You can vary it to speed up
;; or slow down how fast these "move the tape". The GUI offers other
;; methods of adjusting the speed as well.

(set-config-var rf-samples 24000)

;; This sets how long the meters hold peak value.
;; I wish I could tell you what units its in. I can't, though the larger the
;; number, the longer the hold

(set-config-var meter-hold 99999)

;; if you set this to anything other than zero, it will be used
;; by Ardour's metering to define the theoretical maximum sample
;; value that your audio interface could generate/use. the default
;; value (zero) tells Ardour to use a max sample value based on
;; the sample format (16 bit, 32 bit, etc).

(set-config-var max-level 0)

;; this is more or less identical, but defines the minimum level
;; and is used for computing overs. XXX this should be removed,
;; or used for meters as well.

(set-config-var min-level 0)

;;
;; these define the two sounds used for laying click tracks.
;; "click-emphasis-sound" is used for, say, the 4th beat
;; of every 4 beats in 4/4 time, while "click-sound" is
;; used for the rest.
;;
;; the files named here should contain raw (headerless) mono
;; floating point audio data. sox(1) can be used to produce
;; such files from other formats like this:
;;
;;   sox infile.fmt -f -c 1 foo.raw
;;
;; NOTE: there is no provision yet for specifying the sample
;; rate associated with the click sound. So if the files
;; specified are at 48kHz and Ardour is running at 44.1kHz,
;; the clicks will be pitch-shifted, and vice-versa. For now,
;; c'est la vie.

(set-config-var click-sound          "sounds/tabla1.raw")
(set-config-var click-emphasis-sound "sounds/tabla2.raw")
