/*
    Copyright (C) 1999 Paul Barton-Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: audio_clock_ui.cc,v 1.10 2000/10/29 04:42:51 pbd Exp $
*/

#include <gtkmmext/utils.h>
#include <asm/msr.h>

#include "ardour.h"
#include "audio_clock_ui.h"
#include "audiothing.h"
#include "alsa_device.h"
#include "utils.h"

AudioClock_UI::AudioClock_UI (ALSA::MultiChannelDevice *mcdp)
	: colon1 (":"),
	  colon2 (":"),
	  colon3 (":")
{
	mcd = mcdp;

	clock_frame.set_shadow_type (GTK_SHADOW_IN);
	clock_frame.set_name ("BaseFrame");

	pack_start (clock_frame, false, false, 10);

	clock_frame.add (clock_base);
	clock_base.put (label_packer_hbox, 0, 0);

	label_packer_hbox.pack_start (label_packer, true, false);

	label_packer.pack_start (hours_label, false, false);
	label_packer.pack_start (colon1, false, false);
	label_packer.pack_start (minutes_label, false, false);
	label_packer.pack_start (colon2, false, false);
	label_packer.pack_start (seconds_label, false, false);
	label_packer.pack_start (colon3, false, false);
	label_packer.pack_start (frames_label, false, false);

	set_name ("ClockDisplay");
	clock_base.set_name ("ClockDisplay");
	hours_label.set_name ("ClockDisplay");
	minutes_label.set_name ("ClockDisplay");
	seconds_label.set_name ("ClockDisplay");
	frames_label.set_name ("ClockDisplay");
	colon1.set_name ("ClockDisplay");
	colon2.set_name ("ClockDisplay");
	colon3.set_name ("ClockDisplay");

	Gtkmmext::set_usize_to_display_given_text (hours_label,
						    "88", 0, 5);
	Gtkmmext::set_usize_to_display_given_text (minutes_label,
						    "88", 0, 5);
	Gtkmmext::set_usize_to_display_given_text (seconds_label,
						    "88", 0, 5);
	Gtkmmext::set_usize_to_display_given_text (frames_label,
						   "88", 0, 5);

	frames_label.set_alignment (0,0.5);

	clock_running = false;

	last_hrs = 0;
	last_mins = 0;
	last_secs = 0;
	last_frames = 0;
	last_sample = 0;
}

void
AudioClock_UI::connect_to_audiothing (AudioThing *t)

{
	audiothing = t;
	audiothing->going_away.connect 
		(slot (*this, &AudioClock_UI::disconnect_from_audiothing));
	show_display (0, true);
	run_clock ();
}

void
AudioClock_UI::disconnect_from_audiothing (AudioThing *t)

{
	stop_clock ();
	show_display (0, true);
	audiothing = 0;
}

gint
AudioClock_UI::_update (void *arg)

{
	((AudioClock_UI *) arg)->update ();
	return TRUE;
}

void
AudioClock_UI::update ()

{
	size_t current_frame;

 	if (!is_visible()) {
		return;
	}

	if ((current_frame = audiothing->current_frame ()) == last_sample) {
		return;
	}

	show_display (current_frame);
	last_sample = current_frame;
}

void
AudioClock_UI::show_display (size_t sample, bool force)

{
	size_t hrs;
	size_t mins;
	size_t secs;
	size_t frames;
	char buf[8];

	sample_to_smpte_clock_parts (sample,
				     mcd->frame_rate(),
				     Config->smpte_frames_per_second,
				     &hrs,
				     &mins,
				     &secs,
				     &frames);

	if (force || hrs != last_hrs) {
		sprintf (buf, "%02u", hrs);
		hours_label.set_text (buf);
		last_hrs = hrs;
	}

	if (force || mins != last_mins) {
		sprintf (buf, "%02u", mins);
		minutes_label.set_text (buf);
		last_mins = mins;
	}

	if (force || secs != last_secs) {
		sprintf (buf, "%02u", secs);
		seconds_label.set_text (buf);
		last_secs = secs;
	}

	if (force || frames != last_frames) {
		sprintf (buf, "%02u", frames);
		frames_label.set_text (buf);
		last_frames = frames;
	}
}

void
AudioClock_UI::run_clock ()

{
	if (audiothing == 0) {
		return;
	}

	if (!clock_running) {
		clock_timeout_tag = 
			gtk_timeout_add (100, _update, this);
		clock_running = true;
	}
}

void
AudioClock_UI::stop_clock ()

{
	if (!audiothing) {
		return;
	}

	if (clock_running) {
		gtk_timeout_remove (clock_timeout_tag);
		clock_running = false;
	}
}
