# Configure paths for libguileconfig
# Based on libsigc++ script by Karl Nelson

dnl Test for Libguileconfig, and define GUILECONFIG_CFLAGS and GUILECONFIG_LIBS
dnl   to be used as follows:
dnl AM_PATH_GUILECONFIG(MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]])
dnl
AC_DEFUN(AM_PATH_GUILECONFIG,
[
dnl 
dnl Get the cflags and libraries from the guileconfig-config script
dnl

dnl
dnl Prefix options
dnl
AC_ARG_WITH(guileconfig-prefix,
[  --with-guileconfig-prefix=PREFIX
                          Prefix where libguileconfig is installed (optional)]
, guileconfig_config_prefix="$withval", guileconfig_config_prefix="")

AC_ARG_WITH(guileconfig-exec-prefix,
[  --with-guileconfig-exec-prefix=PREFIX 
                          Exec prefix where  libguileconfig is installed (optional)]
, guileconfig_config_exec_prefix="$withval", guileconfig_config_exec_prefix="")

AC_ARG_ENABLE(guileconfigtest, 
[  --disable-guileconfigtest     Do not try to compile and run a test libguileconfig 
                          program],
, enable_guileconfigtest=yes)

dnl
dnl Prefix handling
dnl
  if test x$guileconfig_config_exec_prefix != x ; then
     guileconfig_config_args="$guileconfig_config_args --exec-prefix=$guileconfig_config_exec_prefix"
     if test x${GUILECONFIG_CONFIG+set} != xset ; then
        GUILECONFIG_CONFIG=$guileconfig_config_exec_prefix/bin/guileconfig-config
     fi
  fi
  if test x$guileconfig_config_prefix != x ; then
     guileconfig_config_args="$guileconfig_config_args --prefix=$guileconfig_config_prefix"
     if test x${GUILECONFIG_CONFIG+set} != xset ; then
        GUILECONFIG_CONFIG=$guileconfig_config_prefix/bin/guileconfig-config
     fi
  fi

dnl
dnl See if guileconfig-config is alive
dnl
  AC_PATH_PROG(GUILECONFIG_CONFIG, guileconfig-config, no)
  guileconfig_version_min=$1

dnl
dnl  Version check
dnl
  AC_MSG_CHECKING(for libguileconfig - version >= $guileconfig_version_min)
  no_guileconfig=""
  if test "$GUILECONFIG_CONFIG" = "no" ; then
    no_guileconfig=yes
  else
    guileconfig_version=`$GUILECONFIG_CONFIG --version`

    GUILECONFIG_CFLAGS=`$GUILECONFIG_CONFIG $guileconfig_config_args --cflags`
    GUILECONFIG_LIBS=`$GUILECONFIG_CONFIG $guileconfig_config_args --libs`
    GUILECONFIG_MACROS=`$GUILECONFIG_CONFIG $guileconfig_config_args --libs`

    guileconfig_major_version=`echo $guileconfig_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    guileconfig_minor_version=`echo $guileconfig_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    guileconfig_micro_version=`echo $guileconfig_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`

    guileconfig_major_min=`echo $guileconfig_major_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    guileconfig_minor_min=`echo $guileconfig_minor_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    guileconfig_micro_min=`echo $guileconfig_micro_version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`

    guileconfig_version_proper=`expr \
        $guileconfig_major_version \> $guileconfig_major_min \| \
        $guileconfig_major_version \= $guileconfig_major_min \& \
        $guileconfig_minor_version \> $guileconfig_minor_min \| \
        $guileconfig_major_version \= $guileconfig_major_min \& \
        $guileconfig_minor_version \= $guileconfig_minor_min \& \
        $guileconfig_micro_version \>= $guileconfig_micro_min `

    if test "$guileconfig_version_proper" = "1" ; then
      AC_MSG_RESULT([$guileconfig_major_version.$guileconfig_minor_version.$guileconfig_micro_version])
    else
      AC_MSG_RESULT(no)
      no_guileconfig=yes
    fi

    if test "X$no_guileconfig" = "Xyes" ; then
      enable_guileconfigtest=no
    fi

    AC_LANG_SAVE
    AC_LANG_CPLUSPLUS

dnl
dnl
dnl
    if test "x$enable_guileconfigtest" = "xyes" ; then
      AC_MSG_CHECKING(if libguileconfig sane)
      ac_save_CXXFLAGS="$CXXFLAGS"
      ac_save_LIBS="$LIBS"
      CXXFLAGS="$CXXFLAGS $GUILECONFIG_CFLAGS"
      LIBS="$LIBS $GUILECONFIG_LIBS"

      rm -f conf.guileconfigtest
      AC_TRY_RUN([
#include <stdio.h>
#include <guileconfig/version.h>
#include <pbd/transmitter.h>

Transmitter error (Transmitter::Error);
Transmitter info (Transmitter::Info);
Transmitter warning (Transmitter::Warning);
Transmitter fatal (Transmitter::Fatal);

int main(int argc,char **argv)
  {
   if (guileconfig_major_version!=$guileconfig_major_version ||
       guileconfig_minor_version!=$guileconfig_minor_version ||
       guileconfig_micro_version!=$guileconfig_micro_version)
     { printf("(%d.%d.%d) ",
         guileconfig_major_version,guileconfig_minor_version,guileconfig_micro_version);
       return 1;
     }
  }

],[
  AC_MSG_RESULT(yes)
],[
  AC_MSG_RESULT(no)
  no_guileconfig=yes
]
,[echo $ac_n "cross compiling; assumed OK... $ac_c"])

       CXXFLAGS="$ac_save_CXXFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi

  dnl
  dnl
  if test "x$no_guileconfig" = x ; then
     ifelse([$2], , :, [$2])     
  else
     GUILECONFIG_CFLAGS=""
     GUILECONFIG_LIBS=""
     GUILECONFIG_MACROS=""
     ifelse([$3], , :, [$3])
  fi

  AC_LANG_RESTORE

  AC_SUBST(GUILECONFIG_CFLAGS)
  AC_SUBST(GUILECONFIG_LIBS)
  AC_SUBST(GUILECONFIG_MACROS)
])

