/*
    Copyright (C) 1998 Paul Barton-Davis
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: parser.h,v 1.6 2000/04/08 03:08:35 pbd Exp $
*/

#ifndef  __midi_parse_h__
#define  __midi_parse_h__

#include <vector>
#include <map>
#include <string>

#include <sigc++/signal_system.h>
#include <midi++/types.h>

using namespace std;

class ostream;

namespace MIDI {

class Port;
class Parser;

typedef SigC::Signal2<void, Parser &, byte>                 OneByteSignal;
typedef SigC::Signal2<void, Parser &, EventTwoBytes *>      TwoByteSignal;
typedef SigC::Signal2<void, Parser &, pitchbend_t>          PitchBendSignal;
typedef SigC::Signal3<void, Parser &, byte *, unsigned int> Signal;

class Parser : public SigC::Object {
 public:
	Parser (Port &p);
	~Parser ();
	
	/* signals that anyone can connect to */
	
	OneByteSignal         bank_change;
	TwoByteSignal         note_on;
	TwoByteSignal         note_off;
	TwoByteSignal         poly_pressure;
	OneByteSignal         pressure;
	OneByteSignal         program_change;
	PitchBendSignal       pitchbend;
	TwoByteSignal         controller;

	OneByteSignal         channel_bank_change[16];
	TwoByteSignal         channel_note_on[16];
	TwoByteSignal         channel_note_off[16];
	TwoByteSignal         channel_poly_pressure[16];
	OneByteSignal         channel_pressure[16];
	OneByteSignal         channel_program_change[16];
	PitchBendSignal       channel_pitchbend[16];
	TwoByteSignal         channel_controller[16];
	SigC::Signal1<void, Parser &>          channel_active_preparse[16];
	SigC::Signal1<void, Parser &>          channel_active_postparse[16];

	Signal                raw_preparse;
	Signal                raw_postparse;
	Signal                any;
	Signal                sysex;
	Signal                mmc;
	Signal                position;
	Signal                song;
	SigC::Signal1<void, Parser &>          all_notes_off;
	SigC::Signal1<void, Parser &>          tune;
	SigC::Signal1<void, Parser &>          timing;
	SigC::Signal1<void, Parser &>          start;
	SigC::Signal1<void, Parser &>          stop;
	SigC::Signal1<void, Parser &>          contineu;  /* note spelling */
	SigC::Signal1<void, Parser &>          active_sense;
	SigC::Signal1<void, Parser &>          reset;
	SigC::Signal1<void, Parser &>          eox;

	/* This should really be protected, but then derivatives of Port
	   can't access it.
	*/

	void scanner (byte c);

	size_t *message_counts() { return message_counter; }
	const char *midi_event_type_name (MIDI::eventType);
	void trace (bool onoff, ostream *o, const string &prefix = "");
	Port &port() { return _port; }

	SigC::Signal2<int, byte *, size_t> edit;

	void set_mmc_forwarding (bool yn) {
		_mmc_forward = yn;
	}
	
  private:
	Port &_port;
	/* tracing */

	ostream *trace_stream;
	string trace_prefix;
	void trace_event (Parser &p, byte *msg, unsigned int len);
	SigC::Connection trace_connection;

	size_t message_counter[256];

	enum ParseState { 
		NEEDSTATUS,
		NEEDONEBYTE,
		NEEDTWOBYTES,
		VARIABLELENGTH
	};
	ParseState state;
	unsigned char *msgbuf;
	int msglen;
	int msgindex;
	MIDI::eventType msgtype;
	channel_t channel;
	bool runnable;
	bool _mmc_forward;

	void channel_msg (byte);
	void realtime_msg (byte);
	void system_msg (byte);
	void signal (byte *msg, size_t msglen);
	bool possible_mmc (byte *msg, size_t msglen);
};

}; /* namespace MIDI */

#endif   // __midi_parse_h__

